from PyQt5.QtWidgets import *
from PyQt6.QtWidgets import *
from PySide2.QtWidgets import *
from PySide6.QtWidgets import *

QApplication.exec_ = QApplication.exec

from PyQt6 import QtGui
from PySide6 import QtGui

QAction = QtGui.QAction
QShortcut = QtGui.QShortcut
